@@ -491,6 +491,8 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 	char *str = NULL;
 	char *ptr = NULL;
 	long bugid;
+	char *response;
+	char *tmp;
 	GtkWidget *urlbutton;
 	GtkRequisition requisition;
 	GError *err = NULL;
@@ -508,17 +510,27 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 		remove_pending_reports ();
 	}
 
-	/* parse the XML-RPC response from Bugzilla */
-	bugid = bugzilla_parse_response (msg, &err);
-
-	if (bugid > 0) {
+	/* parse the XML-RPC response */
+	response = bugzilla_parse_response (msg, &err);
+	if (response != NULL) {
+		bugid = strtol (response, &tmp, 10);
 		GtkWidget *main_vbox;
 		
 		/* we need a reference to the vbox containing the text so that we
 	 	* can add a GtkLinkButton to the bug report */
 		main_vbox = GTK_WIDGET (gtk_builder_get_object (ui, "main-vbox"));
 
-		text = g_strdup_printf ("http://bugzilla.gnome.org/show_bug.cgi?id=%ld", bugid);
+		if (response == tmp) {
+			char *url;
+			url = strstr (response, "ViewURL=");
+			if (url) 
+				url += strlen ("ViewURL=");
+			else 
+				url = "";
+
+			text = g_strdup (url);
+		} else 
+			text = g_strdup_printf ("http://bugzilla.gnome.org/show_bug.cgi?id=%ld", bugid);
 
 		/* create a clickable link to the bug report */
 		urlbutton = gtk_link_button_new (text);
@@ -615,6 +627,8 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 	gtk_widget_size_request (GTK_WIDGET (gtk_builder_get_object (ui, "main-window")), &requisition);
 	gtk_window_resize (GTK_WINDOW (gtk_builder_get_object (ui, "main-window")),
 			   requisition.width, requisition.height);
+
+	g_free (response);
 	
 }
 
@@ -742,7 +756,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GtkBuild
 	email = gtk_entry_get_text (GTK_ENTRY (gtk_builder_get_object (ui, "email-entry")));
 	title = create_report_title (app, type, details_text);
 
-	message = bugzilla_create_report (app, type, gnome_version, email, title, final_text, ui, &err);
+	message = bugzilla_create_report (app, type, gnome_version, email, title, final_text, ui, gopt_data.minidump_file, &err);
 	if (message == NULL) {
 		char *text;
 
@@ -823,12 +837,9 @@ email_local_part_is_valid (const char *local_part)
 		return FALSE;
 
 	for (character = local_part; *character; character++) {
-		/* If character is alphanumeric it is valid. */
-		if (g_ascii_isalnum (*character))
-			continue;
-
-		/* If character is "-", "_" or "." it is valid. */
-		if (*character == '-' || *character == '_' || *character == '.')
+		/* RFC 3696 says *any* printable ASCII character can
+		 * appear in local-part, subject to quoting rules. */
+		if (g_ascii_isprint (*character))
 			continue;
 
 		/* Not valid character, not valid local part. */
@@ -903,21 +914,25 @@ email_is_valid (const char *address)
 {
 	char *local_part;
 	char *domain;
-	char **parts;
+	char *at_sign;
 	gboolean is_valid;
 
-	parts = g_strsplit (address, "@", 2);
+	/* Split on the *last* '@' character: */
+	at_sign = strrchr (address, '@');
 
-	/* Check we have the 2 parts */
-	if (!(local_part = parts[0]) || !(domain = parts[1])) {
-		g_strfreev (parts);
+	if (at_sign == NULL) 
 		return FALSE;
-	}
 
+	local_part = g_strndup (address, at_sign - address);
+	domain = g_strdup (at_sign + 1);
+
+	/* Check each part is valid */
 	is_valid = email_local_part_is_valid (local_part)
 		&& email_domain_is_valid (domain);
 
-	g_strfreev (parts);
+	g_free (local_part);
+	g_free (domain);
+
 	return is_valid;
 }
 
@@ -959,6 +974,7 @@ on_send_clicked (GtkWidget *button, gpointer data)
 	send_report (app, gnome_version, ui);
 }
 
+
 static gboolean
 gdb_insert_text (const gchar *stacktrace, GtkBuilder *ui)
 {
@@ -1020,6 +1036,38 @@ create_debuginfo_link (void)
 }
 
 
+static void
+add_minidump_text_output (const char *minidump_file, GtkBuilder *ui)
+{
+	GtkTextView *text_view;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+	gchar *command_line;
+	gchar *standard_output, *standard_error;
+	gint exit_status;
+	GError *error;
+
+
+	text_view = GTK_TEXT_VIEW (gtk_builder_get_object (ui, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+
+	command_line = g_strdup_printf ("minidump_dump %s", minidump_file);
+	if (g_spawn_command_line_sync (command_line, &standard_output, &standard_error,
+				      &exit_status, &error)) {
+		gtk_text_buffer_insert (buffer, &end, standard_output, strlen (standard_output));
+		g_free (standard_output);
+		g_free (standard_error);
+	} else {
+		g_error_free (error);
+	}
+
+}
+
+
+
+
+
 static void
 useless_finished (GtkBuilder *ui)
 {
@@ -1076,7 +1124,7 @@ known_app_finished (GtkBuilder *ui)
 
 	fill_custom_info (app, ui);
 	fill_stderr_info (ui);
-	
+
 	gtk_widget_show (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
 	gtk_widget_show (GTK_WIDGET (gtk_builder_get_object (ui, "send-button")));
 	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "progressbar")));
@@ -1215,6 +1263,8 @@ focus_details (GtkWidget *widget, gpointer data)
 	gtk_widget_grab_focus (widget);
 }
 
+	
+
 static void
 unknown_app_finished (GtkBuilder *ui)
 {
@@ -1222,6 +1272,10 @@ unknown_app_finished (GtkBuilder *ui)
 	char *label_text;
 
 	fill_stderr_info (ui);
+
+	if (gopt_data.minidump_file) {
+		add_minidump_text_output (gopt_data.minidump_file, ui);
+	}
 	
 	/* don't need user input, so hide these widgets */
 	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
@@ -1250,26 +1304,6 @@ unknown_app_finished (GtkBuilder *ui)
 }
 
 
-static void
-send_minidump (GtkBuilder *ui)
-{
-	/* FIXME: Implement the MiniDump sending code */
-	GtkWidget *button;
-
-	button = GTK_WIDGET (gtk_builder_get_object (ui, "close-button"));
-	gtk_button_set_label (GTK_BUTTON (button), _("_Close"));
-	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
-
-	/* don't need user input, so hide these widgets */
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "progressbar")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "send-button")));
-	
-	gtk_label_set_text (GTK_LABEL (gtk_builder_get_object (ui, "main-text")),
-			    _("Your application has crashed. Information about the crash has been successfully collected.\n\n"
-	                      "However we are working on GNOME debug server to handle correctly this information.\n\n"));
-}
-
 
 
 static void
@@ -1809,9 +1843,7 @@ main (int argc, char *argv[])
 	 		* the main loop (and kill gdb) if the user wants to quit before gdb is finished */
 			g_object_set_data (G_OBJECT (ui), "sourceid", GUINT_TO_POINTER (source_id));
 		} else {
-			if (gopt_data.minidump_file != NULL) {
-				send_minidump (ui);
-			} else if (app == NULL) {
+			if (app == NULL) {
 				unknown_app_finished (ui);
 			} else {
 				known_app_finished (ui);
