@@ -24,8 +24,6 @@
 
 #include "bugzilla.h"
 
-#include <glade/glade.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <sys/types.h>
 
 
@@ -48,6 +46,9 @@ typedef struct {
 	/* file to include */
 	gchar *include_file;
 
+	/* MiniDump file */
+	gchar *minidump_file;
+
 	/* pid to kill after the bug report */
 	int kill;
 } GOptionData;
