@@ -32,6 +32,7 @@
 #include <string.h>
 #include <ctype.h>
 
+#include <glib/gi18n.h>
 #include <gnome.h>
 
 #include <gmenu-tree.h>
@@ -65,7 +66,7 @@
 #define BUGZILLA_COMPONENT              "X-GNOME-Bugzilla-Component"
 #define BUGZILLA_EMAIL                  "X-GNOME-Bugzilla-Email"
 #define BUGZILLA_VERSION                "X-GNOME-Bugzilla-Version"
-#define BUGZILLA_OTHER_BINARIES         "X-Gnome-Bugzilla-OtherBinaries"
+#define BUGZILLA_OTHER_BINARIES         "X-GNOME-Bugzilla-OtherBinaries"
 #define BUGZILLA_EXTRA_INFO_SCRIPT      "X-GNOME-Bugzilla-ExtraInfoScript"
 
 #define BA_BUGZILLA_BUGZILLA            "bugzilla:bugzilla"
@@ -111,6 +112,7 @@ add_bugzilla_application (GHashTable *hash,
 			char *s;
 			s = strrchr (programv[0], G_DIR_SEPARATOR);
 			s = s ? s+1 : programv[0];
+			app->ref_count += 1;
 			g_hash_table_insert (hash, g_strdup (s), app);
 		} else {
 			g_free (app);
@@ -123,6 +125,7 @@ add_bugzilla_application (GHashTable *hash,
 	if (other_programs) {
 		programv = g_strsplit (other_programs, ";", -1);
 		for (i=0; programv[i]; i++) {
+			app->ref_count += 1;
 			g_hash_table_insert (hash, g_strdup (programv[i]), app);
 		}
 		g_strfreev (programv);
@@ -132,6 +135,10 @@ add_bugzilla_application (GHashTable *hash,
 static void
 application_free (BugzillaApplication *app)
 {
+	app->ref_count -= 1;
+	if (app->ref_count > 0)
+		return;
+
 	g_free (app->name);
 	g_free (app->cname);
 	g_free (app->comment);
@@ -150,17 +157,16 @@ application_free (BugzillaApplication *app)
 static const GSList *
 get_i18n_slist (void)
 {
-  GList *langs_glist;
-  static GSList *langs_gslist;
+  const char * const *langs;
+  guint i;
+  static GSList *langs_gslist = NULL;
 
   if (langs_gslist)
 	  return langs_gslist;
 
-  langs_glist = (GList *)gnome_i18n_get_language_list ("LC_MESSAGES");
-  langs_gslist = NULL;
-  while (langs_glist != NULL) {
-    langs_gslist = g_slist_append (langs_gslist, langs_glist->data);
-    langs_glist = langs_glist->next;
+  langs = g_get_language_names ();
+  for (i = 0; langs[i] != 0; ++i) {
+	  langs_gslist = g_slist_append (langs_gslist, (gpointer) langs[i]);
   }
 
   return langs_gslist;
@@ -175,7 +181,6 @@ load_applets (GHashTable *hash)
 	GSList *langs;
 	int i;
         gchar *name;
-	GSList *l;
 
 	CORBA_exception_init (&ev);
 	info_list = bonobo_activation_query (APPLET_REQUIREMENTS, NULL, &ev);
@@ -345,7 +350,7 @@ load_applications (void)
 {
 	GSList         *all_applications;
 	GSList         *l;
-	const char     *prev_name = NULL;
+	char           *prev_name = NULL;
 	GError	       *error = NULL;
 
 	GHashTable *program_to_application = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) application_free);
@@ -429,7 +434,8 @@ load_applications (void)
 		g_free (exec);
 		g_free (other_binaries);
 		g_free (extra_info_script);
-		prev_name = gmenu_tree_entry_get_name (entry);
+		g_free (prev_name);
+		prev_name = g_strdup (gmenu_tree_entry_get_name (entry));
 		g_key_file_free (key_file);	
 		gmenu_tree_item_unref (entry);
 	}
@@ -452,27 +458,28 @@ bugzilla_search_for_package (gpointer key, gpointer value, const char *package)
 }
 
 
-int
+char *
 bugzilla_parse_response (SoupMessage *msg, GError **err)
 {
 	SoupXmlrpcResponse *response;
         SoupXmlrpcValue *value;
-	long bugid;
+	long bugid = 0;
 	int debug = 0;
+	char *url;
 
-	g_return_val_if_fail ((err == NULL || *err == NULL), -1);
+	g_return_val_if_fail ((err == NULL || *err == NULL), NULL);
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
 		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_BAD_STATUS, 
 		             _("HTTP Response returned bad status code %d"), msg->status_code);
-		return -1;
+		return NULL;
 	}
 
 	response = soup_xmlrpc_message_parse_response (SOUP_XMLRPC_MESSAGE (msg));
 	if (!response) {
 		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_PARSE_FAILED, 
 		             _("Unable to parse XML-RPC Response")); 
-		return -1;
+		return NULL;
 	}
 
 	/* check to see if the XMLRPC response was a <fault> */
@@ -513,7 +520,7 @@ bugzilla_parse_response (SoupMessage *msg, GError **err)
 		 * construct the GError message  */
 		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_FAULT, 
 		             "%ld:%s", errorcode, errormsg);
-		return -1;
+		return NULL;
 
 	}
 
@@ -524,38 +531,38 @@ bugzilla_parse_response (SoupMessage *msg, GError **err)
 	}
 
 	if (!soup_xmlrpc_value_get_int (value, &bugid)) {
-		debug = 6;
-		goto parse_error;
+		if (!soup_xmlrpc_value_get_string (value, &url)) {
+			debug = 6;
+			goto parse_error;
+		}
 	}	
 
 	/* whew, everything checked out, send back the bug id */
 	g_object_unref (response);
-	return bugid;
+	return bugid ? g_strdup_printf ("%ld", bugid) : url;
 
 parse_error:
 	g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_PARSE_FAILED, 
 	             _("Unable to parse XML-RPC Response\n\n%d\n\n%s"),
 	             debug, soup_xmlrpc_response_to_string (response));
 	g_object_unref (response);
-	return -1;
+	return NULL;
 }
 
 SoupXmlrpcMessage*
 bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gnome_version,
 		        const char *username, const char *title, const char *text,
-		        GladeXML *xml, GError **err)
+		        GtkBuilder *ui, const char *minidump_file, GError **err)
 {
 	SoupXmlrpcMessage *message;
-	char *debug_string;
-	char *distro;
-	char *comment;
 	char *user_agent;
+	char *os_version;
 
 	g_return_val_if_fail (app != NULL, NULL);
 	g_return_val_if_fail (gnome_version != NULL, NULL);
 	g_return_val_if_fail (username != NULL, NULL);
 	g_return_val_if_fail (text != NULL, NULL);
-	g_return_val_if_fail (xml != NULL, NULL);
+	g_return_val_if_fail (ui != NULL, NULL);
 	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
 	/* FIXME: Hardcoded right now */
@@ -571,7 +578,12 @@ bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gn
 		return NULL;
 	}
 
-	message = soup_xmlrpc_message_new ("http://bugzilla.gnome.org/bugbuddy.cgi");
+	if (minidump_file)
+		//message = soup_xmlrpc_message_new ("http://localhost/breakpad/xmlrpc.py");
+		message = soup_xmlrpc_message_new ("http://socorro.gnome.org/collect.py");
+	else 
+		message = soup_xmlrpc_message_new ("http://bugzilla.gnome.org/bugbuddy.cgi");
+
 	if (message == NULL) {
 		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_SEND_ERROR,
 		             _("Unable to create XML-RPC message."));
@@ -606,6 +618,12 @@ bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gn
 	soup_xmlrpc_message_write_string (message, username);
 	soup_xmlrpc_message_end_member (message);
 
+	os_version = get_distro_name ();
+	soup_xmlrpc_message_start_member (message, "os_version");
+	soup_xmlrpc_message_write_string (message, os_version);
+	soup_xmlrpc_message_end_member (message);
+	g_free (os_version);
+
 	if (type == BUG_TYPE_CRASH) {
 		soup_xmlrpc_message_start_member (message, "priority");
 		soup_xmlrpc_message_write_string (message, "High");
@@ -623,6 +641,43 @@ bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gn
 	soup_xmlrpc_message_start_member (message, "comment");
 	soup_xmlrpc_message_write_string (message, text); 
 	soup_xmlrpc_message_end_member (message);
+
+
+	if (minidump_file) {
+		gchar *minidumpbuf;
+		gsize length;
+                GError *error;
+		gchar *base64 = NULL;
+
+		if (g_file_get_contents (minidump_file, &minidumpbuf, &length, &error)) {
+			base64 = g_base64_encode ((guchar*)minidumpbuf, length);
+			g_free (minidumpbuf);
+		} else {
+			g_error_free (error);
+		}
+
+		if (base64) {	
+			gchar *basename;
+			gchar *id;
+
+			basename = g_path_get_basename (minidump_file);
+			id = g_strndup (basename, strlen (basename) - strlen (".dmp"));
+			
+			soup_xmlrpc_message_start_member (message, "minidump-id");
+			soup_xmlrpc_message_write_string (message, id); 
+			soup_xmlrpc_message_end_member (message);
+			
+			soup_xmlrpc_message_start_member (message, "minidump");
+			soup_xmlrpc_message_write_base64 (message, base64, strlen(base64)); 
+			soup_xmlrpc_message_end_member (message);
+			g_free (base64);
+			g_free (basename);
+			g_free (id);
+		}
+	}
+
+
+
 	soup_xmlrpc_message_end_param (message);
 	soup_xmlrpc_message_end_struct (message);
 
