@@ -19,7 +19,14 @@
  */
 
 #include <string.h>
+#include <config.h>
+ 
+#include <glib.h>
+#include <gconf/gconf-client.h>
+ 
+#ifdef ENABLE_EDS
 #include <libebook/e-book.h>
+#endif
 
 #include "eds-buddy.h"
 
@@ -28,8 +35,10 @@ get_default_user_email (void)
 {
 	GConfClient *conf_client;
 	char *tmp;
+#ifdef ENABLE_EDS
 	EBook *book;
 	EContact *me = NULL;
+#endif
 
 	conf_client = gconf_client_get_default ();
 	tmp = gconf_client_get_string (conf_client, "/apps/bug-buddy/email_address", NULL);
@@ -37,7 +46,8 @@ get_default_user_email (void)
 
 	if (tmp != NULL && strlen (tmp) > 0)
 		return tmp;
-		
+
+#ifdef ENABLE_EDS	
 	e_book_get_self (&me, &book, NULL);
 	if (me == NULL)
 		return NULL;
@@ -49,6 +59,7 @@ get_default_user_email (void)
 	tmp = e_contact_get (me, E_CONTACT_EMAIL_1);
 	if (tmp != NULL && strlen (tmp) > 0)
 		return tmp;
+#endif
 
 	return NULL;
 }
